from gi.repository import Gtk, Gio, GtkSource, GObject

class Document(Gtk.ScrolledWindow):
    def __init__(self, filename):
        Gtk.ScrolledWindow.__init__(self)
        
        self.doc = doc = GtkSource.View()
        doc.get_style_context().add_class('white-bg')
        
        doc.set_left_margin(10)
        doc.set_show_line_numbers(False)
        doc.set_tab_width(8)        
        self.add(doc)
        self.lm = lm = GtkSource.LanguageManager()
        self.buffer = buffer = GtkSource.Buffer()
        buffer.set_highlight_syntax(True)
        doc.set_buffer(buffer)
        self.locked = False
        
        self.filename = filename
        self.start_iter = self.buffer.get_start_iter()
        
        if filename == None:
            self.title = 'New Document'
        else:
            content = self.read_content(filename)
            buffer.set_text(content)
            self.detect_language()
            self.detect_title()
        
        
        doc.set_border_window_size(Gtk.TextWindowType.TOP, 20)
        buffer.connect('changed', self._on_changed)
        
    def read_content(self, filename):
        with open(filename, 'r') as f:
            content = f.read()        
        return content
        
    def detect_language(self):
        lang = self.lm.guess_language(self.filename, None)
        self.buffer.set_language(lang)                  
        
    def detect_title(self):
        title = self.strip_title_from_filename(self.filename)
        self.set_title(title)
        
    def auto_save(self):
        self.save_request()
        
    def save_request(self):
        window = self.get_parent().get_parent().get_parent()
        if self.filename != None:
            r = self.save_content()
            window.toolbar.set_save_visible(False)
            return r
        else:            
            d = window.save_dialog()
            if d.run() == 1:
                self.filename = d.get_filename()
                r = self.save_content()
                self.detect_language()
                self.detect_title()
                window.toolbar.set_save_visible(False)
                d.destroy()
                return r
            d.destroy()
        return False
            
    def save_content(self):
        with open(self.filename, 'a') as f:
            content = self.buffer.get_text(self.buffer.get_start_iter(),
                                          self.buffer.get_end_iter(), -1)
            f.seek(0)
            f.write(content)
            return True
        return False
    
    def set_title(self, new_title):
        self.title = new_title
        
    def get_title(self):
        return self.title
        
    def strip_title_from_filename(self, filename):
        title = filename.split('/')[-1]
        return title
        
    def _on_changed(self, data):
        parent = self.get_parent().get_parent().get_parent()
        text_iter = self.buffer.get_end_iter()
        line = text_iter.get_line()
        col = text_iter.get_line_offset()
        parent.bottombar.update_label(line+1, col+1)
        parent.toolbar.set_save_visible(True)
        
    def set_locked(self, val):
        if val == True:
            self.locked = True
            self.doc.set_editable(False)
        else:
            self.locked = False
            self.doc.set_editable(True)
            
    def get_locked(self):
        return self.locked

    def inverse_line_numbers(self):
        r = self.doc.get_show_line_numbers()
        if r == True:
            self.doc.set_show_line_numbers(False)
        else:
            self.doc.set_show_line_numbers(True)
            
    def search(self, text):
        results = []
        found = start_iter.search_forward(text, 0, None)
        if found:
            results.append(found)
            self.start_iter = found[1]
